/**
 *******************************************************************************
 * @file        A33G53x_flash.h
 * @author      ABOV R&D Division
 * @brief       Contains all macro definitions and function prototypes
 *              support for Flash driver on A33G53x
 *
 * Copyright 2023 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

#include "A33G53x.h"
#include "aa_types.h"

//==========================================================================
//
//		D E F I N A T I O N S
//
//==========================================================================
#define DFLASH_ADDR_OFFSET_MASK 		(0x7FFF)

#define A33G538	0x33536000
#define A33G539	0x33538000

#define PAGE_SIZE		0x200
#define SECTOR_SIZE	0x800


//============================================================================
//
//		FLASH ADDRESS
//
//============================================================================
#define CFLASH_BASE_ADDRESS					(0x00000000)

#define CFLASH_SIZE									(0xC0000)			// A33G539
//#define CFLASH_SIZE									(0x80000)		// A33G538


#define DFLASH_BASE_ADDRESS					(0x0F000000)
#define DFLASH_SIZE									(0x8000)

#define READPROTECTION_ADDRESS		0x00200000
#define BANKSWAP_ADDRESS				0x00400000


//==========================================================================
// 	FMCFG
//
//				@ address = 0x4000_0100
//
//==========================================================================
#define FMCFG_DWAIT_VAL(n)				(((n)&0x001FUL)<<8)
#define FMCFG_DWAIT_MASK 				(0x001FUL<<8)

#define FMCFG_CZWAIT					(0x0001UL<<7)

#define FMCFG_CWAIT_VAL(n) 			(((n)&0x001FUL)<<0)
#define FMCFG_CWAIT_MASK 				(0x001FUL<<0)


#define FMCFG_DEFAULT_VAL					(0x0000)
//#define FMCFG_DEFAULT_VAL					(0x0303)




//==========================================================================
// 	FMCON
//
//				@ address = 0x4000_0104
//
//==========================================================================
#define FMCON_RST						(0x0001UL<<31)
#define FMCON_STOP 						(0x0001UL<<30)
#define FMCON_SELF						(0x0001UL<<28)
#define FMCON_BBLK 						(0x0001UL<<21)
#define FMCON_FSRD 						(0x0001UL<<20)
#define FMCON_PAGE 						(0x0001UL<<18)
#define FMCON_TRSL 						(0x0001UL<<17)
#define FMCON_TSMD 						(0x0001UL<<16)
//-----------------------------------------------------------

#define FMCON_DSEL						(0x0001UL<<15)
#define FMCON_CSEL						(0x0001UL<<14)
#define FMCON_AE						(0x0001UL<<13)
#define FMCON_OE						(0x0001UL<<12)
#define FMCON_CS						(0x0001UL<<11)
//-----------------------------------------------------------
#define FMCON_DTBIT						(0x0001UL<<9)
#define FMCON_CTBIT						(0x0001UL<<8)
//-----------------------------------------------------------
#define FMCON_NVSTR					(0x0001UL<<7)
//-----------------------------------------------------------
#define FMCON_DTBIT_SYNC			(0x0001UL<<5)
#define FMCON_CTBIT_SYNC			(0x0001UL<<4)
//-----------------------------------------------------------
#define FMCON_MASE						(0x0001UL<<2)
#define FMCON_SERA						(0x0001UL<<1)
#define FMCON_PROG						(0x0001UL<<0)



//==========================================================================
// 	FMODR
//
//				@ address = 0x4000_0108
//
//==========================================================================

//==========================================================================
// 	FMIDR
//
//				@ address = 0x4000_010C
//
//==========================================================================



//==========================================================================
// 	FMAR
//
//				@ address = 0x4000_0110
//
//==========================================================================
#define FMAR_DCTE 							(0x0001UL<<23)
#define FMAR_LDTE 							(0x0001UL<<22)
#define FMAR_IFREN 							(0x0001UL<<21)
#define FMAR_PRTE 							(0x0001UL<<20)
#define FMAR_REDE 							(0x0001UL<<19)

#define FMAR_ADDR(addr) 					(((addr)&0x0000FFFFUL)<<0)



//==========================================================================
// 	FMTEST
//
//				@ address = 0x4000_0114
//
//==========================================================================
#define FMTEST_WRITE_KEY 					(0x004AUL<<8)
#define FMTEST_EX 							(0x0001UL<<0)


//==========================================================================
// 	FMCRPROT
//
//				@ address = 0x4000_0120
//
//==========================================================================
#define FMC_RPROT 						(0x0039UL)


//==========================================================================
// 	FMCRC
//
//				@ address = 0x4000_0124
//
//==========================================================================
#define FMCRC_CE 						(0x0001UL<<20)
#define FMCRC_CE_RESET					(0x0000<<20)
#define FMCRC_CE_ENABLE				(0x0001<<20)


#define FMCRC_SEL 						(0x0001<<16)
#define FMCRC_SEL_CODE 					(0x0000<<16)
#define FMCRC_SEL_DATA 				(0x0001<<16)


#define FMCRC_CRC16_MASK 				(0xFFFF<<0)


//==========================================================================
// 	FMPROTECT
//
//				@ address = 0x4000_011C
//
//==========================================================================

#define FMPROTECT_BANKSWAP	(0x0001UL<<31)	// Bankswap OTP

#define FMPROTECT_BP11				(0x0001UL<<29)
#define FMPROTECT_BP10				(0x0001UL<<28)
#define FMPROTECT_BP9				(0x0001UL<<23)
#define FMPROTECT_BP8				(0x0001UL<<22)
#define FMPROTECT_BP7				(0x0001UL<<21)
#define FMPROTECT_BP6				(0x0001UL<<20)

#define FMPROTECT_SP7				(0x0001UL<<19)
#define FMPROTECT_SP6				(0x0001UL<<18)
#define FMPROTECT_SP5				(0x0001UL<<17)
#define FMPROTECT_SP4				(0x0001UL<<16)

#define FMPROTECT_DIP					(0x0001UL<<15)
#define FMPROTECT_CIP 				(0x0001UL<<14)	// ReadProtection

#define FMPROTECT_BP5 				(0x0001UL<<13)	// Only 384KB
#define FMPROTECT_BP4 				(0x0001UL<<12)	// Only 384KB

#define FMPROTECT_DP3				(0x0001UL<<11)
#define FMPROTECT_DP2				(0x0001UL<<10)
#define FMPROTECT_DP1				(0x0001UL<<9)
#define FMPROTECT_DP0				(0x0001UL<<8)

#define FMPROTECT_DP_MASK		(0x000FUL<<8)
#define FMPROTECT_DPx_ALL		(0x000FUL<<8)

#define FMPROTECT_BP3 				(0x0001UL<<7)
#define FMPROTECT_BP2 				(0x0001UL<<6)
#define FMPROTECT_BP1 				(0x0001UL<<5)
#define FMPROTECT_BP0 				(0x0001UL<<4)

#define FMPROTECT_SP3 				(0x0001UL<<3)
#define FMPROTECT_SP2 				(0x0001UL<<2)
#define FMPROTECT_SP1 				(0x0001UL<<1)
#define FMPROTECT_SP0 				(0x0001UL<<0)

#define FMPROTECT_CPx_ALL		(0x30FF30FF)


//==========================================================================
// 	FMRPROT
//
//				@ address = 0x4000_0120
//
//==========================================================================
#define FMRPROT_LVL1 					(0x0001UL<<30)

#define FMRPROT_ENTER_LVL1_CODE		(0x0039UL<<0)




//==========================================================================
// 	FMSIZE
//
//				@ address = 0x4000_0128
//
//==========================================================================
#define FMSIZE_256K						(0x000000A5UL)
#define FMSIZE_384K 					(0x0000005AUL)



//==========================================================================
// 	FMBOOT
//
//				@ address = 0x4000_012C
//
//==========================================================================
#define FMBOOT_MAGIC_KEY 				(0x13752451UL)
#define FMBOOT_MAGIC_NULL_KEY 		(0x13752450UL)
#define FMBOOT_BOOT 					(0x0001UL<<0)

//==========================================================================
// 	FMCBCR
//
//				@ address = 0x4000_0154
//
//==========================================================================
#define FMBCR_WRITE_KEY		(0xAA91AUL<<16)
#define FMBCR_BSE				(0x0001UL<<4)
#define FMBCR_BSW				(0x0001UL<<0)

//==========================================================================
// 	FMCBSR
//
//				@ address = 0x4000_0158
//
//==========================================================================
#define FMBSR_CBF				(0x0001UL<<8)
#define FMBCR_BSE				(0x0001UL<<4)
#define FMBCR_BSW				(0x0001UL<<0)

//==========================================================================
// 	FMCECCMR
//
//				@ address = 0x4000_0160
//
//==========================================================================
#define CECCEN					(0x01UL<<0)
#define DECCEN					(0x01UL<<1)

//==========================================================================
// 	FMCIER
//
//				@ address = 0x4000_0180
//
//==========================================================================
#define FMIER_WDEIN			(0x0001UL<<13)
#define FMIER_WEIEN			(0x0001UL<<12)
#define FMIER_BSEN			(0x0001UL<<8)
#define FMIER_DECCIEN		(0x0001UL<<4)
#define FMIER_CECCIEN		(0x0001UL<<0)

//==========================================================================
// 	FMCISR
//
//				@ address = 0x4000_0184
//
//==========================================================================
#define FMISR_WDF				(0x0001UL<<13)
#define FMISR_WEF				(0x0001UL<<12)
#define FMISR_BSERR			(0x0001UL<<8)
#define FMISR_DECCDERR	(0x0001UL<<5)
#define FMISR_DECCSERR	(0x0001UL<<4)
#define FMISR_CECCDERR	(0x0001UL<<1)
#define FMISR_CECCSERR	(0x0001UL<<0)

//=========================================================================
//
//	D E F I N A T I O N S
//
//=========================================================================
#define DFC_CMD_CHIP_ERASE			FMCON_MASE
#define DFC_CMD_SECTOR_ERASE		FMCON_SERA
#define DFC_CMD_PROG_BYTE			FMCON_PROG

#define BANK_DISABLE						DISABLE
#define BANK_ENABLE							FMBCR_BSE
#define BANK_SWAP							(FMBCR_BSE | FMBCR_BSW)

////===============================================================
//// MACROs
////===============================================================


//===============================================================
//
//		F U N C T I O N    D E C L A R A T I O N S
//
//===============================================================
void FLASH_Init (FMC_Type * const flash);
void FLASH_unInit (FMC_Type * const flash);
int FLASH_EraseChip (FMC_Type * const flash, uint32_t addr);
int FLASH_EraseSector (FMC_Type * const flash, uint32_t addr);
int FLASH_Erase4KSector (FMC_Type * const flash, uint32_t addr);
int FLASH_Erase512Page (FMC_Type * const flash, uint32_t addr);
int FLASH_ProgramByte (FMC_Type * const flash, uint32_t addr, uint8_t data);
int FLASH_ProgramWord (FMC_Type * const flash, uint32_t addr, uint32_t data);
uint32_t FLASH_ExecuteFlashOpteration (FMC_Type * const flash, uint32_t cmd, uint32_t addr, uint32_t data, uint32_t limit);

void FLASH_EnableProtection (FMC_Type * const flash, uint32_t mask);
void FLASH_DisableProtection (FMC_Type * const flash, uint32_t mask);
void FLASH_BANKSWAP_Protection (FMC_Type * const flash, FunctionalState endis);
void FLASH_ReadProtection_Protection (FMC_Type * const flash, FunctionalState endis);

int FLASH_Self_EraseSector (FMC_Type * const flash, uint32_t addr);
int FLASH_Self_ErasePage (FMC_Type * const flash, uint32_t addr);
int FLASH_Self_ProgramWORD (FMC_Type * const flash, uint32_t addr, uint32_t data);

void FLASH_ECC_Mode (FMC_Type * const flash, uint8_t Flashmode, FunctionalState endis);
void FLASH_ConfigureInterrupt (FMC_Type * const flash, uint32_t mode, FunctionalState endis);
uint32_t FLASH_Bank_Status (FMC_Type * const flash);
