/**
 *******************************************************************************
 * @file        A33G53x_flash.c
 * @author      ABOV R&D Division
 * @brief       Contains all macro definitions and function prototypes
 *              support for Flash driver on A33G53x
 *
 * Copyright 2023 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/

#include <stdint.h>
#include "A33G53x.h"
#include "A33G53x_flash.h"
#include "aa_types.h"

//#define FLASH_UART_DEBUGGING
#define FLASH_GPIO_DEBUGGING


#ifdef FLASH_UART_DEBUGGING
#include "console.h"
#elif defined(FLASH_GPIO_DEBUGGING)
//#include "hal.h"
#endif



/**
************************************************************************************
* @ Name : FLASH_Init
*
* @ Parameter
*		- flash : FMC
*
*
************************************************************************************
*/
void FLASH_Init (FMC_Type * const flash)
{
	
	return;

}

/**
************************************************************************************
* @ Name : FLASH_unInit
*
* @ Parameter
*		- flash : FMC
*
*
************************************************************************************
*/
void FLASH_unInit (FMC_Type * const flash)
{
	while((FMC->CON & FMCON_CTBIT_SYNC) == FMCON_CTBIT_SYNC);	// Busy Code flag
	while((FMC->CON & FMCON_DTBIT_SYNC) == FMCON_DTBIT_SYNC);	// Busy Data flag
	
	FMC->CON = 0x06000000UL;					// Self-PGM Disable
	FMC->TEST = FMTEST_WRITE_KEY;		// Extended Mode Disable
	
	return;
}



/**
************************************************************************************
* @ Name : FLASH_EraseChip
*
* @ Parameter
*		- flash : FMC
*		- addr : CFLASH_BASE_ADDRESS, DFLASH_BASE_ADDRESS
*
* @ Return
*		0 		success
*		1 		fail
*
*
************************************************************************************
*/
int FLASH_EraseChip (FMC_Type * const flash, uint32_t addr)
{

	uint32_t		limit = 100000;


	limit = FLASH_ExecuteFlashOpteration (FMC, FMCON_MASE, addr, 0, limit);


#ifdef FLASH_UART_DEBUGGING
	cprintf("limit=%d\r\n", limit);
#endif

	if (limit) return (0); 		// success
	else return (1); 			// fail


}



/**
************************************************************************************
* @ Name : FLASH_EraseSector
*
* @ Parameter
*		- flash : FMC
*		- addr : 0x0F00_0000, 0x0F00_0400, 0x0F00_0800, ...0x0F00_7C00
*
* @ Return
*		0 		success
*		1 		fail
*
*
************************************************************************************
* Comments
*
*		This function erases the flasy by 1KB
************************************************************************************
*/
int FLASH_EraseSector (FMC_Type * const flash, uint32_t addr)
{

	uint32_t		limit = 10000;

	limit = FLASH_ExecuteFlashOpteration (FMC, FMCON_SERA, addr, 0, limit);


#ifdef FLASH_UART_DEBUGGING
	cprintf("limit=%d\r\n", limit);
#endif

	if (limit)
		return (0); 		// success
	else
		return (1); 			// fail
}




/**
************************************************************************************
* @ Name : FLASH_Erase512Page
*
* @ Parameter
*		- flash : FMC
*		- addr : 0x0F00_0000, 0x0F00_0400, 0x0F00_0800, ...0x0F00_7C00
*
* @ Return
*		0 		success
*		1 		fail
*
*
************************************************************************************
* Comments
*
*		This function erases the flash by 512B
************************************************************************************
*/
int FLASH_Erase512Page (FMC_Type * const flash, uint32_t addr)
{

	uint32_t		limit = 10000;

	limit = FLASH_ExecuteFlashOpteration (FMC, FMCON_PAGE, addr, 0, limit);

#ifdef FLASH_UART_DEBUGGING
	cprintf("limit=%d\r\n", limit);
#endif

	if (limit) return (0); 		// success
	else return (1); 			// fail
}




/**
************************************************************************************
* @ Name : FLASH_ProgramByte
*
* @ Parameter
*		- flash : FMC
*		- addr : 0x0F00_0000, 0x0F00_0400, 0x0F00_0800, ...0x0F00_7C00
*
* @ Return
*		0 		success
*		1 		fail
*
*
************************************************************************************
* Comments
*
*		If the target flash is the data flash, the size of program-unit is a byte.
************************************************************************************
*/
int FLASH_ProgramByte (FMC_Type * const flash, uint32_t addr, uint8_t data)
{

	uint32_t 		limit = 100000;

	limit = FLASH_ExecuteFlashOpteration(FMC, FMCON_PROG, addr, data, limit);

#ifdef FLASH_UART_DEBUGGING
	cprintf("limit=%d\r\n", limit);
#endif

	if (limit) return (0); 		// success
	else return (1); 			// fail
}

/**
************************************************************************************
* @ Name : FLASH_ProgramWord
*
* @ Parameter
*		- flash : FMC
*		- addr : 0x0F00_0000, 0x0F00_0400, 0x0F00_0800, ...0x0F00_7C00
*
* @ Return
*		0 		success
*		1 		fail
*
*
************************************************************************************
* Comments
*
*		If the target flash is the data flash, the size of program-unit is a word.
************************************************************************************
*/
int FLASH_ProgramWord (FMC_Type * const flash, uint32_t addr, uint32_t data)
{

	uint32_t 		limit = 100000;

	limit = FLASH_ExecuteFlashOpteration(FMC, FMCON_PROG, addr, data, limit);

#ifdef FLASH_UART_DEBUGGING
	cprintf("limit=%d\r\n", limit);
#endif

	if (limit) return (0); 		// success
	else return (1); 			// fail
}

/**
************************************************************************************
* @ Name : FLASH_ExecuteFlashOpteration
*
* @ Parameter
*		- flash : FMC
*		- cmd : FMCON_MASE, "FMCON_MASE+FMCON_BBLK," FMCON_SERA, FMCON_PROG, FMCON_S4K, FMCON_PAGE
*		- addr
*		- data
*		- limit
*
* @ return
*		0 			fail
*		non-zero		The value is the left limit value which is decreased in FMC operation
*
*
************************************************************************************
*/
uint32_t FLASH_ExecuteFlashOpteration (FMC_Type * const flash, uint32_t cmd, uint32_t addr, uint32_t data, uint32_t limit)
{

	volatile uint32_t	delay;
	uint32_t			status;
	uint32_t			code_data_sel, busy_check;

	//-----------------------------------------------------------------------------------
	// examine address
	//
	//						data flash address 		0x0F00_0000 ~ 0x0F00_7FFF
	//-----------------------------------------------------------------------------------
	if (addr < (CFLASH_BASE_ADDRESS+CFLASH_SIZE))
	{
		code_data_sel = FMCON_CSEL;
		busy_check = FMCON_CTBIT_SYNC;
	}
	else if (addr >= DFLASH_BASE_ADDRESS && addr < (DFLASH_BASE_ADDRESS+DFLASH_SIZE))
	{
		code_data_sel = FMCON_DSEL;
		busy_check = FMCON_DTBIT_SYNC;

		addr &= DFLASH_ADDR_OFFSET_MASK;

	}
	else if (addr >= READPROTECTION_ADDRESS && addr <(READPROTECTION_ADDRESS+PAGE_SIZE))
	{
		code_data_sel = FMCON_CSEL;
		busy_check = FMCON_CTBIT_SYNC;
	}
	else if (addr >= BANKSWAP_ADDRESS && addr <(BANKSWAP_ADDRESS+PAGE_SIZE))
	{
		code_data_sel = FMCON_CSEL;
		busy_check = FMCON_CTBIT_SYNC;
	}
	else
	{
		return (0); 		/* FAIL */
	}

	//-----------------------------------------------------------------------------------
	// CSEL OR DSEL
	//-----------------------------------------------------------------------------------
	FMC->CON = 0;
	FMC->CON = code_data_sel;


	//-----------------------------------------------------------------------------------
	// address
	//-----------------------------------------------------------------------------------
	FMC->AR = addr;


	//-----------------------------------------------------------------------------------
	// CS, command
	//-----------------------------------------------------------------------------------
	FMC->CON = (code_data_sel|FMCON_CS|cmd);


	//-----------------------------------------------------------------------------------
	// data
	//-----------------------------------------------------------------------------------
	FMC->ODR = data;


	//-----------------------------------------------------------------------------------
	// AE
	//-----------------------------------------------------------------------------------
	FMC->CON = (code_data_sel|FMCON_CS|cmd|FMCON_AE);


	//-----------------------------------------------------------------------------------
	// NVSTR
	//-----------------------------------------------------------------------------------
	FMC->CON = (code_data_sel|FMCON_CS|cmd|FMCON_AE|FMCON_NVSTR);


	//-----------------------------------------------------------------------------------
	// time delay
	//-----------------------------------------------------------------------------------
	for (delay=0; delay<10; delay++);


	//-----------------------------------------------------------------------------------
	// ~command
	//-----------------------------------------------------------------------------------
	FMC->CON = (code_data_sel|FMCON_CS|FMCON_AE|FMCON_NVSTR);



	//-----------------------------------------------------------------------------------
	// busy check
	//-----------------------------------------------------------------------------------
	while (--limit)
	{
		status = FMC->CON;
		if (!(status & busy_check)) break;
	}



	//-----------------------------------------------------------------------------------
	// ~AE, ~NVSTR
	//-----------------------------------------------------------------------------------
	FMC->CON = (code_data_sel|FMCON_CS);


	//-----------------------------------------------------------------------------------
	// ~CSEL OR ~DSEL, ~CS
	//-----------------------------------------------------------------------------------
	FMC->CON = 0;




	return (limit);
}

/**
************************************************************************************
* @ Name : FLASH_EnableProtection -- DataFlash
*
* @ Parameter
*		- flash : FMC
*		- mask : FMPROTECT_DP3
*				FMPROTECT_DP2
*				FMPROTECT_DP1
*				FMPROTECT_DP0
*				FMPROTECT_DPx_ALL
*
* @ return
*		0 			fail
*		non-zero		The value is the left limit value which is decreased in FMC operation
*
*
************************************************************************************
*/
void FLASH_EnableProtection (FMC_Type * const flash, uint32_t mask)
{

	uint32_t		reg_val;

	reg_val = FMC->PROTECT;
	reg_val &= ~(mask & FMPROTECT_DP_MASK);
	FMC->PROTECT = reg_val;
}

/**
************************************************************************************
* @ Name : FLASH_DisableProtection -- DataFlash
*
* @ Parameter
*		- flash : FMC
*		- mask : FMPROTECT_DP3
*				FMPROTECT_DP2
*				FMPROTECT_DP1
*				FMPROTECT_DP0
*				FMPROTECT_DPx_ALL
*
* @ return
*		0 			fail
*		non-zero		The value is the left limit value which is decreased in FMC operation
*
*
************************************************************************************
*/
void FLASH_DisableProtection (FMC_Type * const flash, uint32_t mask)
{

	uint32_t		reg_val;

	reg_val = FMC->PROTECT;
	reg_val |= (mask & FMPROTECT_DP_MASK);
	FMC->PROTECT = reg_val;
}

/**
************************************************************************************
* @ Name : FLASH_BANKSWAP_Protection
*
* @ Parameter
*		- flash : FMC
 *      - FunctionalState, should be:
*	 					- DISABLE	: 0
*	 					- ENABLE		: 1
*
* @ return
*		0 			fail
*		non-zero		The value is the left limit value which is decreased in FMC operation
*
*
************************************************************************************
*/
void FLASH_BANKSWAP_Protection (FMC_Type * const flash, FunctionalState endis)
{
	uint32_t		reg_val;
	
	reg_val = FMC->PROTECT;
	
	if(endis)
	{		
		reg_val |= FMPROTECT_BANKSWAP;
	}
	else
	{
		reg_val &= ~FMPROTECT_BANKSWAP;
	}
	
	FMC->PROTECT = reg_val;

}

/**
************************************************************************************
* @ Name : FLASH_ReadProtection_Protection
*
* @ Parameter
*		- flash : FMC
 *      - FunctionalState, should be:
*	 					- DISABLE	: 0
*	 					- ENABLE		: 1
*
* @ return
*		0 			fail
*		non-zero		The value is the left limit value which is decreased in FMC operation
*
*
************************************************************************************
*/
void FLASH_ReadProtection_Protection (FMC_Type * const flash, FunctionalState endis)
{
	uint32_t		reg_val;
	
	reg_val = FMC->PROTECT;
	
	if(endis)
	{	
		reg_val |= FMPROTECT_CIP;
	}
	else
	{
		reg_val &= ~FMPROTECT_CIP;
	}
	
	FMC->PROTECT = reg_val;

}

/**
************************************************************************************
* @ Name : Flash Memoery Code ECC Mode Register
*
* @ Parameter
*		- flash : FMC
*		- Flashmode : Function mode, should be:
* 					- CODE_FLASH	: 0
* 					- DATA_FLASH		: 1
*      - FunctionalState, should be:
*	 					- DISABLE	: 0
*	 					- ENABLE		: 1
*
* @ return
*		0 			fail
*		non-zero		The value is the left limit value which is decreased in FMC operation
*
*
************************************************************************************
*/
void FLASH_ECC_Mode (FMC_Type * const flash, uint8_t Flashmode, FunctionalState endis)
{
	uint32_t		reg_val;
	
	reg_val = FMC->ECCMR;
	
	if(Flashmode == CECCEN)
	{
		if(endis)
		{
			reg_val |= CECCEN;
		}
		else
		{
			reg_val &= ~CECCEN;
		}		
	}
	else if(Flashmode == DECCEN)
	{
		if(endis)
		{
			reg_val |= DECCEN;
		}
		else
		{
			reg_val &= ~DECCEN;
		}		
	}
	
	FMC->ECCMR = reg_val;

}

/**
************************************************************************************
* @ Name : Flash Memoery Interrupt Enable Register
*
* @ Parameter
*		- flash : FMC
*		- mode : Flash Interrupt mode, should be:
* 					- FMIER_WDEIN
* 					- FMIER_WEIEN
* 					- FMIER_BSEN
* 					- FMIER_DECCIEN
* 					- FMIER_CECCIEN
*      - FunctionalState, should be:
*	 					- DISABLE	: 0
*	 					- ENABLE		: 1
*
* @ return
*		0 			fail
*		non-zero		The value is the left limit value which is decreased in FMC operation
*
*
************************************************************************************
*/
void FLASH_ConfigureInterrupt (FMC_Type * const flash, uint32_t mode, FunctionalState endis)
{
	uint32_t		reg_val;
	
	reg_val = FMC->IER;
	
	if(endis)
	{
		reg_val |= mode;
	}
	else
	{
		reg_val &= ~mode;
	}		
	
	FMC->IER = reg_val;
}

/**
************************************************************************************
* @ Name : Flash Memoery Interrupt Enable Register
*
* @ Parameter
*		- flash : FMC
*		
*
* @ return
*		uint32_t 	result
*		- FMBCR_BSE
*		- FMBCR_BSW
*
************************************************************************************
*/
uint32_t FLASH_Bank_Status (FMC_Type * const flash)
{
	uint32_t		reg_val;
	
	reg_val = FMC->BSR;
	if((reg_val & BANK_SWAP) == BANK_SWAP)
	{
		reg_val = BANK_SWAP;
	}
	else if((reg_val & BANK_SWAP) == BANK_ENABLE)
	{
		reg_val = BANK_ENABLE;
	}
	else if((reg_val & BANK_SWAP) == BANK_DISABLE)
	{
		reg_val = BANK_DISABLE;
	}
	
	return reg_val;	
}


/**
************************************************************************************
* @ Name : FLASH_Self_EraseSector
*
* @ Parameter
*		- flash : FMC
*		- addr : 0, 0x400, ...
*
*
* @ return
*		0 			fail
*		non-zero		The value is the left limit value which is decreased in FMC operation
*
*
************************************************************************************
*
* @ Location of the operating code : Code Flash
*
************************************************************************************
*/
int FLASH_Self_EraseSector (FMC_Type * const flash, uint32_t addr)
{

	volatile int		delay = 100000;
	uint32_t	status=0;

	FMC->CON = (FMCON_SELF|0x06000000UL|FMCON_SERA);

	*((volatile uint32_t *) addr) = 0;

	while (--delay)
	{
		status = FMC->CON;
		if (!(status & FMCON_CTBIT_SYNC)) break;
	}

	return (0);

}

/**
************************************************************************************
* @ Name : FLASH_Self_ErasePage
*
* @ Parameter
*		- flash : FMC
*		- addr : 0, 0x400, ...
*
*
* @ return
*		0 			fail
*		non-zero		The value is the left limit value which is decreased in FMC operation
*
*
************************************************************************************
*
* @ Location of the operating code : Code Flash
*
************************************************************************************
*/
int FLASH_Self_ErasePage (FMC_Type * const flash, uint32_t addr)
{

	volatile int		delay = 100000;
	uint32_t	status=0;

	FMC->CON = (FMCON_SELF|0x06000000UL|FMCON_PAGE);

	*((volatile uint32_t *) addr) = 0;

	while (--delay)
	{
		status = FMC->CON;
		if (!(status & FMCON_CTBIT_SYNC)) break;
	}

	return (0);

}

/**
************************************************************************************
* @ Name : FLASH_Self_ProgramWORD
*
* @ Parameter
*		- flash : FMC
*		- addr : 0, 0x400, ...
*		- data
*
*
* @ return
*		0 			fail
*		non-zero		The value is the left limit value which is decreased in FMC operation
*
*
************************************************************************************
*
* @ Location of the operating code : Code Flash
*
************************************************************************************
*/
int FLASH_Self_ProgramWORD (FMC_Type * const flash, uint32_t addr, uint32_t data)
{

	volatile int		delay = 100000;
	uint32_t	status=0;

	FMC->CON = (FMCON_SELF|0x06000000UL|FMCON_PROG);

	*((volatile uint32_t *) addr) = data;

	while (--delay)
	{
		status = FMC->CON;
		if (!(status & FMCON_CTBIT_SYNC)) break;
	}

	return (0);

}


