/**
 *******************************************************************************
 * @file        A33G53x_pmu.c
 * @author      ABOV R&D Division
 * @brief       Contains all macro definitions and function prototypes
 *              support for PMU driver on A33G53x
 *
 * Copyright 2023 ABOV Semiconductor Co.,Ltd. All rights reserved.
 *
 * This file is licensed under terms that are found in the LICENSE file
 * located at Document directory.
 * If this file is delivered or shared without applicable license terms,
 * the terms of the BSD-3-Clause license shall be applied.
 * Reference: https://opensource.org/licenses/BSD-3-Clause
 ******************************************************************************/
 

/* Includes ------------------------------------------------------------------- */
#include <stdint.h>
#include "A33G53x.h"
#include "A33G53x_pmu.h"
#include "A33G53x_pcu.h"
#include "A33G53x_gpio.h"
//#include "A33G53x_flash.h"
#include "debug_frmwrk.h"


extern uint32_t		SystemCoreClock;
extern uint32_t		SystemPeriClock;


#if 0
/**********************************************************************
 * @brief 		Configure external sub clock
 *             Before exteranl crystal enable setting, you have to set clock ports as SXIN,SXOUT in MOD,AFSRx reg
 * @param[in]	esosccon 	Function mode, should be:
 * 					- EXSOSC_STOP	   : 0
 * 					- EXSOSC_EN    	   : 8
 * 					- EXSOSC_EN_DIV2 : 9
  * 				- EXSOSC_EN_DIV2 : 10
 * @return		None
 **********************************************************************/
void PMU_SetSOSC(uint32_t esosccon)
{
	uint32_t reg_val32;

// enable external sub clock
// Before exteranl crystal enable setting, you have to set clock ports as SXIN,SXOUT in MOD,AFSRx reg
	reg_val32 = PMU->CCR;
	reg_val32 &= ~(0x03 << 2); //
	reg_val32 |= (esosccon << 2); // 0:Stop SOSC, 8:Enable SOSC/1, 9:Enable SOSC/2, 10:Enable SOSC/4
	PMU->CCR = reg_val32;
}

/**********************************************************************
 * @brief 	  monitoring External sub oscillator
 * @param  none
 * @return	status enumeration value:
 *              - ERROR
 *              - SUCCESS
 **********************************************************************/
Status SCU_WaitForSOSCStartUp(void)
{
//	__IO uint32_t StartUpCounter = 0;
//	__IO uint16_t status;
	uint32_t i;

	// monitoring External sub oscillator
//	SCU->CMR |= (1<<11); // 0:External sub oscillator monitoring disabled, 1:External sub oscillator monitoring enable

	/* Wait till SOSC is ready and if timeout is reached exit */
//	do
//	{
//		status = (SCU->CMR & SCU_CMR_SOSCSTS_Msk);
//		StartUpCounter++;
//	} while((StartUpCounter != SOSC_STARTUP_TIMEOUT) && (status != SCU_CMR_SOSCSTS_Msk));

	/* need more Wait for SOSC stable, check SXOUT pin oscillation  */
	for (i=0;i<0x100;i++)
	{
		__NOP();
	}

//	status = (SCU->CMR & SCU_CMR_SOSCSTS_Msk);
//	if (status != SCU_CMR_SOSCSTS_Msk){
//		return ERROR;
//	}
//	else {
		return SUCCESS;
//	}
}

/**********************************************************************
 * @brief 		Configure low speed internal osc clock
 * @param[in]	ringosc Function mode, should be:
 * 					- LSIOSC_STOP        : 0
 * 					- LSIOSC_EN          : 8
 * 					- LSIOSC_EN_DIV2     : 9
 * 					- LSIOSC_EN_DIV4     : 10
 * @return		None
 **********************************************************************/
void SCU_SetLSIOSC(uint32_t ringosc)
{
	uint32_t reg_val32;

// enable internal clock
	reg_val32 = PMU->CCR;
	reg_val32 &= ~(0x03 << 6); //
	reg_val32 |= (ringosc<<6); // 0:Stop LSI, 8:Enable LSI/1, 9:Enable LSI/2, 10:Enable LSI/4
	PMU->CCR = reg_val32;
}

/**********************************************************************
 * @brief 		Configure high speed internal osc clock
 * @param[in]	hsiosc Function mode, should be:
 * 					- HSIOSC_STOP        : 0
 * 					- HSIOSC_EN             : 8
 * 					- HSIOSC_EN_DIV2     : 9
 * 					- HSIOSC_EN_DIV4     : 10
 * 					- HSIOSC_EN_DIV8     : 11
 * 					- HSIOSC_EN_DIV16    : 12
 * 					- HSIOSC_EN_DIV32    : 13
 * @return		None
 **********************************************************************/
void SCU_SetHSIOSC(uint32_t hsiosc)
{
	uint32_t reg_val32;

// enable internal clock
	reg_val32 = PMU->CCR;
	reg_val32 &= ~(0x03 << 4);
	reg_val32 |= (hsiosc << 4); // 0:Stop LSI, 8:Enable LSI/1, 9:Enable LSI/2, 10:Enable LSI/4, 11:Enable LSI/8, 12:Enable LSI/16, 13:Enable LSI/32
	PMU->CCR = reg_val32;
}

/**********************************************************************
 * @brief 		Configure external clock
 *             Before exteranl crystal enable setting, you have to set clock ports as XIN,XOUT in MR.
 * @param[in]	eosccon 	Function mode, should be:
 * 					- EXOSC_STOP        : 0
 * 					- EXOSC_EN             : 8
 * 					- EXOSC_EN_DIV2     : 9
 * 					- EXOSC_EN_DIV4     : 10
 * @return		None
 **********************************************************************/
void SCU_SetMOSC(uint32_t eosccon)
{
	uint32_t reg_val32;

// enable external clock
// Before exteranl crystal enable setting, you have to set clock ports as XIN,XOUT in MOD,AFSRx reg
	reg_val32 = PMU->CCR;
	reg_val32 &= ~(0x03 << 0); //
	reg_val32 |= (eosccon << 0); // 0:Stop MOSC, 8:Enable MOSC/1, 9:Enable MOSC/2, 10:Enable MOSC/4
	PMU->CCR = reg_val32;
}

/**********************************************************************
 * @brief 		Clock Monitoring Enable
 *             Set Clock Monitoring Function.
 * @param[in]	eosccon 	Function mode, should be:
 * 					- EXOSC_STOP        : 0
 * 					- EXOSC_EN             : 8
 * 					- EXOSC_EN_DIV2     : 9
 * 					- EXOSC_EN_DIV4     : 10
 * @return		None
 **********************************************************************/
void SCU_SetClockMonitoring(uint32_t enable)
{
	PMU->CMR = enable;
}

/**********************************************************************
 * @brief 	  monitoring External oscillator
 * @param  none
 * @return	status enumeration value:
 *              - ERROR
 *              - SUCCESS
 **********************************************************************/
Status SCU_WaitForMOSCStartUp(void)
{
	__IO uint32_t StartUpCounter = 0;
	__IO uint16_t status;
	uint32_t i;

	// monitoring External oscillator
	PMU->CMR |= 0x0101; // 0:External oscillator monitoring disabled, 1:External oscillator monitoring enable

	/* Wait till MOSC is ready and if timeout is reached exit */
	do {
		status = (PMU->CMR & PMU_CMR_MXOSCSTS);
		StartUpCounter++;
	} while ((StartUpCounter < MOSC_STARTUP_TIMEOUT) && (status != PMU_CMR_MXOSCSTS));

	/* need more Wait for MOSC stable, check XOUT pin oscillation  */
	for (i = 0 ; i < 0x800 ; i++) {
		__NOP();
	}

	status = (PMU->CMR & PMU_CMR_MXOSCSTS);
	if (status != PMU_CMR_MXOSCSTS){
		return ERROR;
	} else {
		return SUCCESS;
	}
}

/**********************************************************************
 * @brief 		change system clock control
 * @param[in]	sysclkcon mode, should be:
 * 					- SC_LSIOSC		         : 0x00
 * 					- SC_SOSC  	             : 0x01
 * 					- SC_HSIOSC  	         : 0x03
 * 					- SC_HSIOSCPLL           : 0x02
 * 					- SC_EXOSC               : 0x13
 * 					- SC_EXOSCPLL            : 0x12
 * @return		None
 **********************************************************************/
void SCU_ChangeSysClk(uint32_t sysclkcon)
{
	uint32_t reg_val32;
//	int i;

	reg_val32 = PMU->BCCR;
	reg_val32 &= ~(0x1F << 0); //
	reg_val32 |= sysclkcon;
	PMU->BCCR = reg_val32;

//	for(i=0; i < 1000; i++);			// delay for clock stabilization

}


/**********************************************************************
 * @brief 		Configure clock out  on PF4
 *       Before this function setting, you have to set PF4 as CLKO in PF.MODand PF.AFSR0.
 * @param[in]	divval(=CLKODIV) : 0~15  // CLKO=MCLK/(2*(CLKODIV+1))
 *                   ex) MCLK=20Mh, CLKODIV=4,    20Mhz/ (2*(4+1)) = 2Mhz
 * @param[in]	FunctionalState, should be:
 * 					- DISABLE		   : 0
 * 					- ENABLE    	   : 1
 * @return		None
 **********************************************************************/
void SCU_SetCOR(uint8_t divval, FunctionalState endis)
{
	PMU->COR = (endis<<4)|(divval&0xf);
}
#endif

/**
CLOCK_DELAY value must be changed according to the system clock used by the user.
1. LSI1MHz		: 0x40 
2. HSE8MHz : 0x200
3. HSI16MHz	: 0x400
*/

/* LSI 1MHz - 1ms */
#define CLOCK_DELAY	0x40

/**********************************************************************
 * @brief 	  monitoring External oscillator
 * @param  none
 * @return	status enumeration value:
 *              - ERROR
 *              - SUCCESS
 **********************************************************************/
Status SCU_SetPLLandWaitForPLLStartUp(FunctionalState pllsetstate, uint8_t selbypass, uint8_t selfin,  uint8_t PREDIV, uint8_t POSTDIV1, uint8_t POSTDIV2, uint8_t OUTDIV)
{
	__IO uint32_t StartUpCounter = 0;
	__IO uint32_t status;
//	uint32_t i;

	PMU->PLLCON = 0x80750000;			// Key value for setting PLL Frequency (Extended Mode)

	if (pllsetstate == DISABLE){
		PMU->PLLCON = 0;
		return SUCCESS;
	} else {
// PLLenable    PLL freq = (FIN or FIN/2) * M / N
		PMU->PLLCON = 0
		|(1 << 15)   // PLL reset is negated
		|(1 << 14)  // PLLEn
		|((selbypass & 1) << 13)     	//BYPASS 0:FOUT is bypassed as FIN, 1:FOUT is PLL output
//		|((selfin & 1) << 20)           //PLL VCO mode  0:VCO is the same with FOUT, 1:VCO frequency is x2 of FOUT   D
		|((PREDIV & 0x7) << 8)      	//PREDIV        R
		|((POSTDIV1 & 0xff) << 20)    	//POSTDIV1   N1 = MULT
		|((POSTDIV2 & 0xf) << 16)     	//POSTDIV2   N2
		|((OUTDIV & 0xf) << 0)         	//OUTDIV	     P
			;

//		PMU->PLLCON&=~(1<<23);   // PLL reset is asserted
//		for (i=0;i<1000;i++);
//		PMU->PLLCON|=(1<<23);     // PLL reset is negated

	  /* Wait till PLL LOCK is ready and if timeout is reached exit */
		do
		{
			status = (PMU->PLLCON & PLLCON_LOCKSTS);
			StartUpCounter++;
		} while ((StartUpCounter != PLL_STARTUP_TIMEOUT) && (status != PLLCON_LOCKSTS));

		status = (PMU->PLLCON & PLLCON_LOCKSTS);
		if (status != PLLCON_LOCKSTS){
			return ERROR;
		}
		else {
			return SUCCESS;
		}
	}
}

/**
************************************************************************************
* @ Name : PMU_SetPLLFreq
*
* @ Parameters
*
*		- pmu				PMU
*		- pllInClk 			XTAL4MHz, XTAL8MHz, XTAL10MHz, IOSC16MHz
*		- setPLLFreq		PLL75MHz, PLL74MHz, ..., PLL1MHz
*
* @ Description : this function helps users easilly set the value of PLL operating frequency.
************************************************************************************
*/
int32_t PMU_SetPLLFreq(PMU_Type *pmu, uint32_t pllInClk, uint32_t setPllFreq) {
//	uint32_t	reg_val;

	uint32_t r      = 0;							// PLL input clock divider (R)
	uint32_t n1 	= 0;							// PLL multiplyer (N1)
	uint32_t n2 	= 0;							// PLL divider (N2)
	uint32_t p 		= 0;							// PLL post-divider (P)
	uint32_t d		= 0;							// PLL VCO doubler bit value (D)
	uint32_t f_in   = 0;							// PLL input clock freq.
	uint32_t f_vco  = 0;							// PLL VCO freq.


	FMC->CFG = 0x0303;						// Set Flash Access Timing (Max 75MHz/(2+3) = 15.00MHz)


	if(pllInClk == XTAL4MHz || pllInClk == XTAL8MHz ||  pllInClk == XTAL10MHz) {        
        PMU_SetHSE(PMU_CCR_MXOSCEN_DIV_BY_1);

    //	for (delay = 0; delay < 0x100 ; delay++) { }    /*< delay stabilization (200 ms) @ 75 MHz */
        PMU_WaitForHSEStartUp();

        PMU_PLLINCLKCmd(PMU_BCCR_PLLCLKSEL_XTAL);
        PMU_MainCLKCmd(PMU_BCCR_MCLKSEL_PLL_BYPASS);
        PMU_PLLPREDIVCmd(PMU_BCCR_PLLCLKDIV_DIV_BY_1);

        PMU_HCLKDIVCmd(PMU_BCCR_HCLKDIV_DIV_BY_1);     // HCLK = MCLK/1
        PMU_PCLKDIVCmd(PMU_BCCR_PCLKDIV_DIV_BY_1);     // PCLK = HCLK/1
            
        SystemCoreClock = pllInClk * 1000000UL; 	   /*< Update System Core Clock (HCLK) */
        SystemPeriClock = pllInClk * 1000000UL; 	   /*< Update System Peri Clock (PCLK) */   
        
	}    
    
	else if (pllInClk == IOSC16MHz) {
        PMU_SetHSI(PMU_CCR_IOSC16EN_DIV_BY_1);

        PMU_PLLINCLKCmd(PMU_BCCR_PLLCLKSEL_IOSC16M);
        PMU_MainCLKCmd(PMU_BCCR_MCLKSEL_PLL_BYPASS);
        
        PMU_HCLKDIVCmd(PMU_BCCR_HCLKDIV_DIV_BY_1);     // HCLK = MCLK/1
        PMU_PCLKDIVCmd(PMU_BCCR_PCLKDIV_DIV_BY_1);     // PCLK = HCLK/1

        SystemCoreClock = pllInClk * 1000000UL; 	   /*< Update System Core Clock (HCLK) */
        SystemPeriClock = pllInClk * 1000000UL; 	   /*< Update System Peri Clock (PCLK) */
	}
	else {
		return PLL_WRONG;
	}

	// Set Deviders
	if(pllInClk == XTAL4MHz) 			r = 1;							// FIN = 4/(1+1) == 2
	else if(pllInClk == XTAL8MHz)		r = 3;							// FIN = 8/(1+3) == 2
	else if(pllInClk == XTAL10MHz)		r = 4;							// FIN = 10/(1+4) == 2
	else if(pllInClk == IOSC16MHz)		r = 7;							// FIN = 16/(1+7) == 2
	else return PLL_WRONG;

	f_in = pllInClk/(r+1);												// f_in is fixed to 2MHz.

	if(f_in == 2) {
		if( (setPllFreq <= 75) && (setPllFreq >= 26)) {
			n1 = setPllFreq-1;
			n2 = 0;
			p = 1;
			d = 0;
		} else if( (setPllFreq <= 25) && (setPllFreq >= 11)) {
			n1 = (setPllFreq*3)-1;
			n2 = 0;
			p = 5;
			d = 0;
		} else if( (setPllFreq <= 10) && (setPllFreq >= 4)) {
			n1 = (setPllFreq*8)-1;
			n2 = 0;
			p = 15;
			d = 0;
		} else if( (setPllFreq <= 3) && (setPllFreq >= 2)) {
			n1 = (setPllFreq*16)-1;
			n2 = 1;
			p = 15;
			d = 0;
		} else if(setPllFreq == 1) {
			n1 = (setPllFreq*32);
			n2 = 3;
			p = 15;
			d = 0;
		} else {
			return PLL_WRONG;
		}

		f_vco = f_in * n1;
		if( f_vco <= 200) {
			PMU_PLLEnable (PMU, r, n1, n2, p, d);       /*< Set PLL VCO */
		}
	}
	else {
		return	PLL_WRONG;
	}

    PMU_MainCLKCmd(PMU_BCCR_MCLKSEL_PLL);       /*< Set PLL to MCLK */    
    
	SystemCoreClock = (setPllFreq * 1000000);
	SystemPeriClock =  (setPllFreq * 1000000);

	return PLL_OK;
}

/**
************************************************************************************
* @ Name : PMU_PLLEnable
*
* @ Parameters
*
*		- pmu			PMU
*		- prediv 			0~7
*		- mul			(0~255
*		- div			0~15
*		- postdiv			0~15
*		- vco_mode 		PLLCON_VCOMODE_NORMAL, PLLCON_VCOMODE_DOUBLE
*
************************************************************************************
*/

void PMU_PLLEnable (PMU_Type *pmu, uint32_t prediv, uint32_t mul, uint32_t div, uint32_t postdiv, uint32_t vco_mode)
{
    uint32_t            count = 0;
    
    // Key value for setting PLL Frequency (Extended Mode)
	PMU->PLLCON = 0x80750000;
    
	// PLL Setting
	PMU->PLLCON = 0    
                | PLLCON_PLLEN    
                | PLLCON_PLLnRESB
                | PLLCON_BYPASS                 // (nBYPASS, 0: PLL, 1: PLLINCLK)    
                | PLLCON_PREDIV_VAL(prediv)     // (R)
                | PLLCON_MULTI_VAL(mul)         // (N1)
                | PLLCON_DIV_VAL(div)           // (N2)
                | PLLCON_POSTDIV_VAL(postdiv)   // (P)
                | vco_mode                      // (D)
                ;

		/* wait over 500us */
		if(PMU->BCCR & PMU_BCCR_PLLCLKSEL_MASK)
			count = PLL_STARTUP_TIMEOUT;	// HSE - 8MHz
		else 
			count = PLL_STARTUP_TIMEOUT * 2;	// HSI - 16MHz	
		while(--count);

		/* Check the PLLCON PLLLOCK State */
		while((PMU->PLLCON & (1UL<<12)) == 0 )
			__NOP();		
}


/**
************************************************************************************
* @ Name : PMU_ConfigureInterrupt
*
* @ Parameter
*		pmu			= PMU
*
*		intr_mask	= PMUCMR_SXOSCIE, PMUCMR_MXOSCIE, (PMUCMR_SXOSCIE|PMUCMR_MXOSCIE)
*
*		enable		= INTR_ENABLE, INTR_DISABLE
*
*
************************************************************************************
*/
void PMU_ConfigureInterrupt (PMU_Type *pmu, uint32_t intr_mask, uint32_t enable)
{
	uint32_t		reg_val;

	reg_val = PMU->CMR;
	reg_val &= ~(PMU_CMR_MXOSCIE); 							//disable interrupt

	PMU->CMR = (reg_val|(PMU_CMR_MXOSCIF)); 			// clear interrupt flag

	if (enable)
	{
		reg_val |= (intr_mask & (PMU_CMR_MXOSCIE)); 	// enable interrupt
		PMU->CMR = reg_val;
	}
}

/**
************************************************************************************
* @ Name : PMU_CheckResetEvent
*
* @ Parameter
*		pmu			= PMU
*
* @ Function
*		Print Reset Event
************************************************************************************
*/
void PMU_CheckResetEvent (PMU_Type *pmu)
{
	uint32_t		reg_val;

	reg_val = (pmu->RSSR);
	cprintf("\r\n\r\n[0x%08X] Reset Event by ", reg_val);

	if((reg_val&PMU_RSSR_MCKFAILRST) == PMU_RSSR_MCKFAILRST)
		cprintf("/Main Clock Failure ");
	if((reg_val&PMU_RSSR_RSTRST) == PMU_RSSR_RSTRST)
		cprintf("/nReset");
	if((reg_val&PMU_RSSR_SYSRST) == PMU_RSSR_SYSRST)
		cprintf("/Core Reset Request ");
	if((reg_val&PMU_RSSR_SWRST) == PMU_RSSR_SWRST)
		cprintf("/Software Request ");
	if((reg_val&PMU_RSSR_WDTRST) == PMU_RSSR_WDTRST)
		cprintf("/Watchdog Reset ");
	if((reg_val&PMU_RSSR_MXFAILRST) == PMU_RSSR_MXFAILRST)
		cprintf("/Main X-tal OSC Failure ");
	if ((reg_val&PMU_RSSR_LVDRST) == PMU_RSSR_LVDRST)
		cprintf("/LVD Reset ");
	if(reg_val == 0x0)
		cputs("/Power On Reset\r\n");

	pmu->RSSR = 0;
	cputs("\r\n");
}

/**********************************************************************
 * @brief 		Main(System) Clock Selection  
 * @param[in]	clock Function mode, should be:
* 					- PMU_BCCR_MCLKSEL_RING_OSC : LSI
* 					- PMU_BCCR_MCLKSEL_SUB_OSC : LSE
* 					- PMU_BCCR_MCLKSEL_PLL : PLL
* 					- PMU_BCCR_MCLKSEL_PLL_BYPASS : PLL Bypass (HSE, HSI)
 *
 * @return		None
 **********************************************************************/
void PMU_MainCLKCmd (uint8_t MCLKSRC)
{	
    PMU->BCCR &= ~(PMU_BCCR_MCLKSEL_MASK);
    PMU->BCCR |= (MCLKSRC & PMU_BCCR_MCLKSEL_MASK);
}

/**********************************************************************
 * @brief 		PCLK Clock Divider
 * @param[in]	clock Function mode, should be:
* 					- PMU_BCCR_PCLKDIV_DIV_BY_1: PCLK = HCLK
* 					- PMU_BCCR_PCLKDIV_DIV_BY_2: PCLK = HCLK/2
 *
 * @return		None
 **********************************************************************/
void PMU_PCLKDIVCmd (uint32_t PCLKDIV)
{	
    PMU->BCCR &= ~(PMU_BCCR_PCLKDIV_MASK);
    PMU->BCCR |= (PCLKDIV & PMU_BCCR_PCLKDIV_MASK);
}

/**********************************************************************
 * @brief 		HCLK Divider Selection  
 * @param[in]	clock Function mode, should be:
 * 					- PMU_BCCR_HCLKDIV_DIV_BY_1	: HCLK = MCLK
 * 					- PMU_BCCR_HCLKDIV_DIV_BY_2	: HCLK = MCLK/2
 * 					- PMU_BCCR_HCLKDIV_DIV_BY_4	: HCLK = MCLK/4
 * @return		None
 **********************************************************************/
void PMU_HCLKDIVCmd (uint32_t HCLKDIV)
{
    PMU->BCCR &= ~(PMU_BCCR_HCLKDIV_MASK);
    PMU->BCCR |= (HCLKDIV & PMU_BCCR_HCLKDIV_MASK);
}

/**********************************************************************
 * @brief 		PLL Input Clock Selection  
 * @param[in]	clock Function mode, should be:
 * 					- PMU_BCCR_PLLCLKSEL_IOSC16M  : HSI
 * 					- PMU_BCCR_PLLCLKSEL_XTAL     : HSE
 * @return		None
 **********************************************************************/
void PMU_PLLINCLKCmd (uint32_t PLLINSRC)
{
    PMU->BCCR &= ~(PMU_BCCR_PLLCLKSEL_MASK);
    PMU->BCCR |= (PLLINSRC & PMU_BCCR_PLLCLKSEL_MASK);
}

/**********************************************************************
 * @brief 		PLL Input Clock Divider  
 * @param[in]	clock Function mode, should be:
* 					- PMU_BCCR_PLLCLKDIV_DIV_BY_1 : f_PLLINCLK/1
* 					- PMU_BCCR_PLLCLKDIV_DIV_BY_2 : f_PLLINCLK/2
 * @return		None
 **********************************************************************/
void PMU_PLLPREDIVCmd (uint8_t PLLPREDIV)
{
    PMU->BCCR &= ~(PMU_BCCR_PLLCLKDIV_MASK);
    PMU->BCCR |= (PLLPREDIV & PMU_BCCR_PLLCLKDIV_MASK);
}

/******************************************************************************
* @brief      Control LSI (Low Speed Internal oscillator)
* @param      LSI_Control, should be:
*              - PMU_CCR_ROSCEN_STOP        : Stop LSI
*              - PMU_CCR_ROSCEN_DIV_BY_1    : LSI/1
*              - PMU_CCR_ROSCEN_DIV_BY_2    : LSI/2
* @return     None
*******************************************************************************/
void PMU_SetLSI(uint32_t LSI_Control) {
    uint32_t reg_val32;
    uint32_t i;

    reg_val32 = PMU->CCR;
    reg_val32 &= ~(PMU_CCR_ROSCEN_MASK);
    reg_val32 |= (LSI_Control & PMU_CCR_ROSCEN_MASK);
    PMU->CCR = reg_val32;

    /* about 1ms delay, The delay value may very depending on the user's circuit enviroment. */
    for(i=0;i<CLOCK_DELAY;i++);
}

/******************************************************************************
* @brief      Control LSE (Low Speed External oscillator)
* @param      LSE_Control, should be:
*              - PMU_CCR_SXOSCEN_STOP        : Stop LSI
*              - PMU_CCR_SXOSCEN_DIV_BY_1    : LSE/1
*              - PMU_CCR_SXOSCEN_DIV_BY_2    : LSE/2
* @return     None
*******************************************************************************/
void PMU_SetLSE(uint32_t LSE_Control) {
    uint32_t reg_val32;
	uint32_t i;
	
    // SXOUT, SXIN Pin configuration
	PCU_ConfigureFunction(PCE, PIN_8, PE8_MUX_SXIN); 
	PCU_SetDirection(PCE, PIN_8, ANALOG_INPUT);      
    
	PCU_ConfigureFunction(PCE, PIN_9, PE9_MUX_SXOUT); 
	PCU_SetDirection(PCE, PIN_9, ANALOG_INPUT);        
    
    // Set LSE
    reg_val32 = PMU->CCR;
    reg_val32 &= ~(PMU_CCR_SXOSCEN_MASK);
    reg_val32 |= (LSE_Control & PMU_CCR_SXOSCEN_MASK);
    PMU->CCR = reg_val32;

	/* Need more Wait for LSE stable, check LSE pin oscillation.
				about 1s delay, The delay value may vary depending on the user's circuit environment. */
	for(i=0;i<CLOCK_DELAY*1000;i++);	
}

/******************************************************************************
* @brief      Control HSE (HIgh Speed External oscillator)
* @param      HSE_Control, should be:
*              - PMU_CCR_MXOSCEN_STOP        : Stop LSI
*              - PMU_CCR_MXOSCEN_DIV_BY_1    : HSE/1
*              - PMU_CCR_MXOSCEN_DIV_BY_2    : HSE/2
* @return     None
*******************************************************************************/
void PMU_SetHSE(uint32_t HSE_Control) {
    uint32_t reg_val32;
	uint32_t i;
	
    // XTALO, XTALI Pin configuration
	PCU_ConfigureFunction(PCC, PIN_14, PC14_MUX_XTALO); 
	PCU_SetDirection(PCC, PIN_14, ANALOG_INPUT);      
    
	PCU_ConfigureFunction(PCC, PIN_15, PC15_MUX_XTALI); 
	PCU_SetDirection(PCC, PIN_15, ANALOG_INPUT);

    // Set HSE
    reg_val32 = PMU->CCR;
    reg_val32 &= ~(PMU_CCR_MXOSCEN_MASK);
    reg_val32 |= (HSE_Control & PMU_CCR_MXOSCEN_MASK);
    PMU->CCR = reg_val32;

	/* Need more Wait for HSE stable, check HSE pin oscillation.
				about 10ms delay, The delay value may vary depending on the user's circuit environment. */
	for(i=0;i<CLOCK_DELAY*10;i++);	
}

/******************************************************************************
* @brief      Control HSI (How Speed Internal oscillator)
* @param      HSI_Control, should be:
*              - PMU_CCR_IOSC16EN_STOP      : Stop HSI
*              - PMU_CCR_IOSC16EN_DIV_BY_1  : HSI/1
*              - PMU_CCR_IOSC16EN_DIV_BY_2  : HSI/2
* @return     None
*******************************************************************************/
void PMU_SetHSI(uint32_t HSI_Control) {
    volatile uint32_t reg_val32;
    uint32_t i;

    reg_val32 = PMU->CCR;
    reg_val32 &= ~(PMU_CCR_IOSC16EN_MASK);
    reg_val32 |= (HSI_Control & PMU_CCR_IOSC16EN_MASK);
    PMU->CCR = reg_val32;

    /* about 1ms delay, The delay value may very depending on the user's circuit enviroment. */
    for(i=0;i<CLOCK_DELAY;i++);
}

/**********************************************************************
 * @brief 		Configure clock out  on PC13
 *       Before this function setting, you have to set PC13 as CLKO in PC_MR and PC_CR
 * @param[in]	sel, should be:
 *                  - PMU_COR_CLKOSEL_PLL : CLKO uses PLL
 *                  - PMU_COR_CLKOSEL_MCLK : CLKO uese MCLK
 * @param[in]	FunctionalState, should be:
 * 					- PMU_COR_CLKOEN_CLKO_OFF	: Disable CLKO
 * 					- PMU_COR_CLKOEN_CLKO_ON    : Enable CLKO
 * @param[in]	CLKODIV(=CLKODIV) : 0 to 15  // CLKO = MCLK/(2*(CLKODIV))
 *                   ex) MCLK=10MHz, CLKODIV=5,    10MHz/ (2*5)) = 1MHz
 * @return		None
 **********************************************************************/
void PMU_SetCOR(uint8_t sel, uint8_t endis, uint8_t CLKODIV)
{
	PCU_ConfigureFunction(PCC, PIN_13, PC13_MUX_CLKO); 
	PCU_SetDirection(PCC, PIN_13, PUSHPULL_OUTPUT);

	PMU->COR &= ~( PMU_COR_CLKOSEL | PMU_COR_CLKOEN | PMU_COR_CLKODIV_MASK );
    PMU->COR |= ((sel & PMU_COR_CLKOSEL)| (endis & PMU_COR_CLKOEN) | PMU_COR_CLKODIV_VAL(CLKODIV));
}


void PMU_WaitForHSEStartUp(void) {
	uint32_t count;
    
	// Monitoring HSE
    PMU->CMR |= (PMU_CMR_MXOSCMNT | PMU_CMR_MXOSCSTS);    /* Clear MXOSCSTS flag & Enable MXOSC (HSE) Monitoring */

    /*< Delay 200 ms (8 MHz, 3-wait) */
    for (count = 0 ; count < 0x19000; count++) {
        /*< Exit the loop when the MOSC status bit is flagged. */
        if ((PMU->CMR & PMU_CMR_MXOSCSTS) == PMU_CMR_MXOSCSTS) {
            break;
        } else {
            __NOP();
        }
    }
}


/* --------------------------------- End Of File ------------------------------ */
